
import globals

from putil import *

from Image import Image
import pygame
from math import *
from Map import Map

from OpenGL.GL import *
from OpenGL.GLU import *

class Cart:
    def __init__(self, startpoint=(0,0), map= None):
        
        self.cartStraight = Image('cart1-forward')
        self.cartLeft = Image('cart1-15left')
        self.cartLeftTight = Image('cart1-25left')
        self.cartRight = Image('cart1-15right')
        self.cartRightTight = Image('cart1-25right')

        self.cartStraightb = Image('cart1-forward-b')
        self.cartLeftb = Image('cart1-15left-b')
        LeftTightb = Image('cart1-25left-b')
        self.cartRightb = Image('cart1-15right-b')
        self.cartRightTightb = Image('cart1-25right-b')
        
        self.imagesa = []
        self.imagesa.append(Image('cart1-forward'))
        self.imagesa.append(Image('cart1-15right'))
        self.imagesa.append(Image('cart1-25right'))
        self.imagesa.append(Image('cart1-right'))
        self.imagesa.append(Image('cart1-frontleft'))
        self.imagesa.append(Image('cart1-front'))
        self.imagesa.append(Image('cart1-frontright'))
        self.imagesa.append(Image('cart1-left'))
        self.imagesa.append(Image('cart1-25left'))
        self.imagesa.append(Image('cart1-15left'))

        self.imagesb = []
        self.imagesb.append(Image('cart2-back'))
        self.imagesb.append(Image('cart2-right15'))
        self.imagesb.append(Image('cart2-right25'))
        self.imagesb.append(Image('cart2-right'))
        self.imagesb.append(Image('cart2-frontleft'))
        self.imagesb.append(Image('cart2-front'))
        self.imagesb.append(Image('cart2-frontright'))
        self.imagesb.append(Image('cart2-left'))
        self.imagesb.append(Image('cart2-left25'))
        self.imagesb.append(Image('cart2-left15'))

        self.gdot = Image('ping')
        
        self.cartImage = self.cartStraight
        
        self.dirtImage = Image('dirt')
        self.map = map
        self.size = 20
        self.radius = self.size
        self.mass = 10
        self.turnSpeed = 75.0
        self.minTurnSpeed = 20.0
        self.x, self.y = startpoint
        self.x *= map.scale
        self.y *= map.scale
        self.xv = 0
        self.yv = 0
        self.steam = 50
        self.lap = 0
        self.fire = 50
        self.velocity = 0
        self.throttle = 'n' # n == neutral, f == forward, r == reverse
        self.rotation = map.startrotation
        self.wheels = 's' # s == straight, l == left, r == right
        self.dir = 1.0
        self.slide = False
        self.eBreak = False
        self.slideTime = 0
        self.dir_map = 0
        self.start_time = pygame.time.get_ticks()
        self.lap_times = []
        
        self.direction = self.rotation
        
        self.momentum = 'n' # n == stoped, f == forward, r == reversed
        self.breaking = False
        
        #print "Initializing cart: ", self.x, self.y, self.rotation
        
        
    def draw(self):
        cw = 80
        ch = cw / 2.0
        
        #self.gdot.draw(pos=(self.x-self.radius,self.y-self.radius),width=self.radius*2, height=self.radius*2)
        
        glPushMatrix()
        glTranslate(self.x, self.y, 16)
        glRotate(-self.rotation,0,0,-1)
        glTranslate(-1 * cw/2.0, -1 * ch/2.0, 0)
        
        glRotate(-1.0*(globals.firstIsoRotation + globals.secondIsoRotation), 1.0, 0, 0)
        
        oc = 1.0
        if self.fire > 100:
            oc = (1000 - self.fire) / 900.0
            
        if self.breaking:
            if self.wheels == 'l':
                self.cartImage = self.cartLeftb
            elif self.wheels == 'r':
                self.cartImage = self.cartRightb
            else:
                self.cartImage = self.cartStraightb
        else:
            if self.wheels == 'l':
                if self.eBreak:
                    self.cartImage = self.cartLeftTight
                else:
                    self.cartImage = self.cartLeft
            elif self.wheels == 'r':
                if self.eBreak:
                    self.cartImage = self.cartRightTight
                else:
                    self.cartImage = self.cartRight
            else:
                self.cartImage = self.cartStraight        
            

        self.cartImage.draw(width = cw, height = ch, color=(1.0, oc, oc, 1.0))
        if self.slide or self.eBreak:
            glDisable(GL_DEPTH_TEST)
            #self.dirtImage.drawIso(-1.0*self.rotation, width = cw, height = 16)
            self.dirtImage.draw(width = cw, height = 16)
            glEnable(GL_DEPTH_TEST)
        glPopMatrix()

    
    def update(self, dt):
  
        
        self.direction = self.rotation
        
        old_pos = (self.x, self.y)
        in_the_zone = 0  # flag for finish line test
        
        self.physUpdate(dt)
        
        if self.slideTime > 0:
            self.slideTime -= dt
        if self.slideTime < 0:
            self.slideTime = 0
        
        
        
        if self.map.startline1[1] > self.map.startline2[1]:
            if old_pos[1]/self.map.scale < self.map.startline1[1] and old_pos[1]/self.map.scale > self.map.startline2[1]:
                in_the_zone = 1
        else:
            if old_pos[1]/self.map.scale < self.map.startline2[1] and old_pos[1]/self.map.scale > self.map.startline1[1]:
                in_the_zone = 1
                
        if in_the_zone:
            if old_pos[0]/self.map.scale < self.map.startline2[0] and self.x/self.map.scale >= self.map.startline2[0]:
                self.lap += 1
                self.lap_times.append((self.lap,int(pygame.time.get_ticks()-self.start_time)))
            if old_pos[0]/self.map.scale >= self.map.startline2[0] and self.x/self.map.scale < self.map.startline2[0]:
                self.lap -=1

        # steam engine physics
        coolrate = 5
        heatfactor = 60
        throttlefactor = 8
        
        if self.fire > 0:
            self.fire -= coolrate * dt
            if self.fire < 0:
                self.fire = 0
        self.steam += self.fire/heatfactor * dt
        
        if self.throttle != 'n':
            self.steam -= self.steam/throttlefactor * dt
            
        if self.lap >=3:
            return (1, self.lap_times)
        else:
            return (0, 0)
        

    def physUpdate(self, dt):
        throttleVelocity = 1000.0
        slideSpeedTreshold = 50.0
        slideAngleTreshold = 5.0
        
        slideFriction = 400.0
        normalFriction = 100.0
        roughtFrictionMultiplierSlow = 1
        roughtFrictionMultiplierFast = 40
        roughFrictionSpeedTreshold = 0.3
        
        maxVelocity = 1200.0
        maxVelocityBackwards = 600.0
        
        breakingForce = 10.0
        

        
        # if we are just starting to move, flag our momentum.
        if self.yv == 0 and self.xv == 0:
            if self.throttle == 'f':
                self.momentum = 'f'
                
            elif self.throttle == 'r':
                self.momentum = 'r'
                
            else:
                self.momentum = 'n'
        



        # calculate the magnitude of our speed vector
        currspeed = (self.xv ** 2 + self.yv ** 2) ** 0.5
        
                
        # check if we are breaking
        if currspeed != 0:
            if self.throttle != 'n':
                if self.momentum != self.throttle:
                    self.breaking = True
                else:
                    self.breaking = False
            else:
                self.breaking = False
                
        
        if self.momentum == 'r':
            rotationDirection = -1.0
        else:
            rotationDirection = 1.0
        
        turn = self.turnSpeed * currspeed/maxVelocity
        if turn < self.minTurnSpeed:
            turn = self.minTurnSpeed
        
        # steer
        if self.wheels != 's':
            if self.wheels == 'l':
                self.rotation += turn * dt * rotationDirection
            else:
                self.rotation -= turn * dt * rotationDirection
                

            if self.rotation >= 360:
                self.rotation -= 360
            elif self.rotation <= -360:
                self.rotation += 360              

            
        # set our velocity if the throttle is open and we are not on the breaks
        if self.throttle != 'n' and not self.breaking:
            if self.throttle == 'f' and currspeed < maxVelocity: # and self.momentum != 'r':
                self.velocity = throttleVelocity
            elif self.throttle == 'r' and currspeed < maxVelocityBackwards:
                self.velocity = -throttleVelocity
            else:
                self.velocity = 0
        else:
            self.velocity = 0
        
        
        dxv = sin(radians(self.rotation))
        dyv = cos(radians(self.rotation))
        
        if self.slide or self.eBreak:
            self.xv += dxv * self.velocity * dt
            self.yv += dyv * self.velocity * dt
            
            if self.yv != 0 and self.slideTime == 0:
                slideAngle = degrees(atan(self.xv/self.yv))

                if currspeed < slideSpeedTreshold:
                    self.slide = False
                elif (slideAngle + slideAngleTreshold > self.rotation) and (slideAngle - slideAngleTreshold < self.rotation):
                    self.slide = False
        else: # direct drive phisics
            if currspeed > 0:  # if we are moving
                if self.momentum == 'f': # forward
                    self.xv = currspeed * dxv
                    self.yv = currspeed * dyv
                elif self.momentum == 'r':
                    self.xv = -1.0 * currspeed * dxv
                    self.yv = -1.0 * currspeed * dyv

            self.xv += dxv * self.velocity * dt
            self.yv += dyv * self.velocity * dt
            

        if self.slide or self.eBreak:
            friction = slideFriction
        else:
            friction = normalFriction
            
        if self.breaking:
            friction *= breakingForce


        #todo: make real collision detection
        tileType = self.map.tileType((self.x, self.y), 1)
        if tileType == "wall":
            #print "Tile (",self.x/self.map.scale,",",self.y/self.map.scale,") is on: ", self.map.tileType((self.x, self.y), 1)
            """self.yv *= -0.5
            self.xv *= -0.5
            self.slide = True"""
        elif tileType == "rough": # and not (self.slide or self.eBreak):
            if currspeed < roughFrictionSpeedTreshold * maxVelocity:
                friction *= roughtFrictionMultiplierSlow
            else:
                friction *= roughtFrictionMultiplierFast
        elif tileType == "hazard":
            self.yv = 0
            self.xv = 0
            if self.fire > 50:
                self.fire = 50


        if self.xv > 0:
            self.xv -= friction * dt
            if self.xv < 0:
                self.xv = 0                
        elif self.xv < 0:
            self.xv += friction * dt
            if self.xv > 0:
                self.xv = 0                
        if self.yv > 0:
            self.yv -= friction * dt
            if self.yv < 0:
                self.yv = 0
        elif self.yv < 0:
            self.yv += friction * dt
            if self.yv > 0:
                self.yv = 0

            
        if (self.xv ** 2 + self.yv ** 2) ** 0.5 < 800 * dt: # and (self.xv != 0 or self.yv !=0):
            self.xv = 0
            self.yv = 0
            self.breaking = False
            self.momentum = 'n'
        else:
            self.x -= self.xv * dt #* self.velocity
            self.y += self.yv * dt #* self.velocity
            
        
        """Make friction a function of dxv and dyv?"""
        
        
       
            
            
        #Hit the outer edge
        if self.x < self.map.min_x*self.map.scale:
            self.x = self.map.min_x*self.map.scale
            self.xv *= -1.0
            self.slide = True
        elif self.x > self.map.max_x*self.map.scale - self.size: 
            self.x = self.map.max_x*self.map.scale - self.size 
            self.xv *= -1.0
            self.slide = True
        if self.y < self.map.min_y*self.map.scale:
            self.y = self.map.min_y*self.map.scale
            self.yv *= -1.0
            self.slide = True
        elif self.y > self.map.max_y*self.map.scale - self.size:
            self.y = self.map.max_y*self.map.scale - self.size
            self.yv *= -1.0
            self.slide = True
            
            
    def getImage(self, angle, playerNum):
        
        delta = self.direction - angle
        
        if (delta > 360):
            delta -= 360
        
        #print "Player angle = ", angle," and I'm looking at ",self.direction, " detla = ", delta
        img = 0
        
        swap = False
        if (delta < 0):
            delta = delta * -1
            swap = True
            
        if (-5 <= delta <= 5):
            img = 0
        elif (6 <= delta <= 25):
            img = 1
        elif (26 <= delta <= 65):
            img = 2
        elif (66 <= delta <= 115):
            img = 3
            #if (swap):
            #    img = 7
        elif (116 <= delta <= 150):
            img = 4
        elif (151 <= delta <= 210):
            img = 5
        elif (211 <= delta <= 240):
            img = 6
        elif (241 <= delta <= 310):
            img = 7
            #if (swap):
            #    img = 3
        elif (311 <= delta <= 320):
            img = 8
        elif (321 <= delta <= 350):
            img = 9
            
        """
        elif (-6 >= delta >= -25):
            img = 1
        elif (-26 >= delta >= -65):
            img = 2
        elif (-66 >= delta >= -115):
            img = 7
        elif (-116 >= delta >= -150):
            img = 8
        elif (-151 >= delta >= -210):
            img = 0
        elif (-211 >= delta >= -240):
            img = 6
        elif (-241 >= delta >= -310):
            img = 9
        elif (-311 >= delta >= -320):
            img = 8
        elif (-321 >= delta >= -350):
            img = 9
        """
            
        if (playerNum == 1):
            return self.imagesa[img]
        else:
            return self.imagesb[img]

    def move(self, pos):
        self.x, self.y = pos
    
    def addFire(self):
        self.fire += 25
        
    def hasInteresect(self, A, B, C, D):
        #print "check"
        xa, ya = A
        xb, yb = B
        xc, yc = C
        xd, yd = D
    
        denom = (yb-ya+0.0)*(xc-xd)-(yd-yc)*(xa-xb)
        if denom == 0:
            return 0
        
        x = ((xa-xb+0.0)*(xd*yc-xc*yd)-(xc-xd)*(xb*ya-xa*yb)) / denom
        
        if xa > xb:
            if x < xa and x > xb:
                return 1
        else:
            if x < xb and x > xa:
                return 1
                
        return 0