import sys
import os
import gutil
import pygame
from pygame.locals import *
from OpenGL.GL import *
from Text import Text
from Image import Image
from OpenGL.GLU import *


#Makes a pretty screen and scrolls credits around and around
class CreditRoll:

    # ttl   = seconds before returning if no key is pressed. -1 is infinate
    # file  = file to read from
    # speed = credits per minute to scroll through
    def __init__(self, screen, ttl=-1, file="credits.txt", speed="10", size="50"):
        self.w = screen[0]
        self.h = screen[1]
        self.size =  int(size)
        self.speed = int(speed)
        self.text_list = self.load(file)
        self.gl_lists = []
        self.displayList, self.width, self.height, self.number = self.generateDisplayList()
        self.white = (255,255,255,255)
        self.background = Image("Racer2")
        self.y_offset = 0
        pygame.mixer.music.stop()
        pygame.mixer.music.load(os.path.join('data', "creditmusic.ogg"))
        pygame.mixer.music.play()
        
    #Loads a credit file and returns a list of strings
    def load(self, filename):
        file = open(filename, "r")
        text = file.read()
        
        lines = text.split("\n")    
        return lines
    
    #Makes a display list with all the names
    def generateDisplayList(self):
        padding = 10
        y_offset = 0
        max_width = 0
        white = (0,0,0,255)
        
        #load all the lines into segments and load those into the main list
        #grab the width at the same time so we can center all purdy like  
        for x in self.text_list:
            temp = Text(x,color=white, fontsize = self.size)
            self.gl_lists.append(temp)
            
            if temp.width > max_width:
                max_width = temp.width
                
        newList = glGenLists(1)
        glNewList(newList, GL_COMPILE)
        #loop through again and throw into the display list
        for credit in self.gl_lists:
            x_offset = (max_width - credit.width)/2
            y_offset -= credit.height + padding
        
            glPushMatrix()            
            glTranslate(x_offset, y_offset, 0)
            glCallList(credit.displayList)
            glPopMatrix()
        glEndList()
        
        return newList, max_width, y_offset, len(self.gl_lists)
        
        
    #draws the list to the screen on a vertical offset   
    def draw(self, pos=(0,0), color=(1,1,1,1), screen_width=800):
        glDisable(GL_DEPTH_TEST)
        glMatrixMode(GL_PROJECTION)
        glPushMatrix()
        glLoadIdentity()
        gluOrtho2D(0, self.w, 0, self.h)
        glMatrixMode(GL_MODELVIEW)
        glPushMatrix()
        glColor4fv(color)


        self.background.draw(color = (255,255,255,128), width = self.w, height = self.h)
        x_offset = (screen_width - self.width)/2

        if pos:
            glTranslate(pos[0]+x_offset,pos[1]+self.y_offset,0)

        
        glCallList(self.displayList)

        glPopMatrix()
        glMatrixMode(GL_PROJECTION)
        glPopMatrix()
        glMatrixMode(GL_MODELVIEW)
        glEnable(GL_DEPTH_TEST)
        
    #updates the current delta
    def update(self, dt):

        self.y_offset -= self.height*self.speed*dt/60
        if self.y_offset >= -self.height+800:
            self.y_offset = 0
            
            
    def handleKey(self, type, key):
        if type == KEYDOWN:
            if key == K_ESCAPE:
                return (1,1,0)
            elif key == K_SPACE:
                return (1,1,0)
        return (0,0,0)

        
#solo command line just pukes back all the lines in the file
def main(args):
    credits = CreditRoll()
    cfg = credits.load(args[1])
    for x in cfg:
        if not x:
            x = " <-- Empty --> "
        print "--->",x,"<---\n"
        
    sys.exit(0)
     
if __name__ == "__main__":
    main(sys.argv)