import gutil
import pygame
import os

import globals

from random import *
from math import *
from pygame.locals import *
from OpenGL.GL import *
from OpenGL.GLU import *

from putil import *

from Cart import Cart
from Image import Image
from Text import Text
from Map import Map
from HUD import HUD
from EffectList import EffectList
from EffectSteam import EffectSteam
from Animation import Animation

from ComputerOpponent import ComputerOpponent

class Engine:

    def __init__(self, screen, map_name, mode='f',player=1,computer=False):
        self.w = screen[0]
        self.h = screen[1]
        
        self.playerNum = player
        
        self.computer = computer
        if computer:
            self.map = Map(map_name, needs_path = True)
        else:
            self.map = Map(map_name)
        self.scale = self.map.map_scale
        self.cow = Image('cow')
        self.cart = Animation(['roller1'])
        self.dirt = Image('dirt')
        if player==1:
            self.player = Cart(map = self.map, startpoint=self.map.startpoint)
            self.x, self.y = self.map.startpoint
        else:
            self.player = Cart(map = self.map, startpoint=(self.map.startpoint[0]-10, self.map.startpoint[1]-10))
            self.x, self.y = self.map.startpoint[0]-10, self.map.startpoint[1]-10
         
        self.computerPlayers = []
        if self.computer:
            for i in range(computer):
                computerPlayer = Cart(map = self.map, startpoint=(self.map.startpoint[0]-10, self.map.startpoint[1]-10))
                opponent = ComputerOpponent(computerPlayer, self.map)
                self.computerPlayers.append(opponent)
                
        self.background = []
        
        for i in range(8):
            r = randint(1,11)
            img = Image(str(r))
            x = randint(0,self.map.width)*self.map.scale*1.0
            y = 0
            temp = (img, y, x)
            self.background.append(temp)
        for i in range(8):
            r = randint(1,11)
            img = Image(str(r))
            x = randint(0,self.map.width)*self.map.scale*1.0
            y = self.map.height*self.map.scale*1.0
            temp = (img, y, x)
            self.background.append(temp)
        for i in range(8):
            r = randint(1,11)
            img = Image(str(r))
            x = 0
            y = randint(0,self.map.height)*self.map.scale*1.0
            temp = (img, y, x)
            self.background.append(temp)
        for i in range(8):
            r = randint(1,11)
            img = Image(str(r))
            x = self.map.width*self.map.scale*1.0
            y = randint(0,self.map.height)*self.map.scale*1.0
            temp = (img, y, x)
            self.background.append(temp)
        
        self.rotation = 0
        self.hud = HUD(self.w,self.h,self.player, self.map.minimap)
        pygame.mixer.music.stop()
        pygame.mixer.music.load(os.path.join('data', "RaceBeginning.ogg"))
        pygame.mixer.music.play()
        self.start_time = pygame.time.get_ticks()
        self.music = 1
        self.effects = EffectList()
        self.mode = mode
        self.player_list = [((self.x/self.map.scale, self.y/self.map.scale), (0,0,1,1))]
        #self.event_capture = []
        #self.capture = open("capture.txt", "w")  
        #self.capture.write("#This file is automatically generated.\n")
        
        #trying to move stuff out of gutil
        self.glSetup()
        
        self.setKeys(player)
        
        
        self.boxList = []
        
        myScale = self.map.scale * self.map.fudge
        
        for x, y in self.map.wallpoints:
            #self.boxList.append((x*self.map.scale*4, y*self.map.scale, self.map.fudge*self.map.scale, self.map.fudge*self.map.scale))
            self.boxList.append((x*myScale, y*myScale, self.map.fudge*myScale/2.0, self.map.fudge*myScale/2.0))

        
    def __del__(self):
        self.glTearDown()

    def glSetup(self):
        
        glMatrixMode(GL_PROJECTION)
        glPopMatrix()
        glPushMatrix()

        if self.mode != 'f':
            if self.mode == 't':
                glViewport(0,self.h/2,self.w,self.h/2)
            else:
                glViewport(0,0,self.w,self.h/2)
            gluPerspective(70.0, (self.w + 0.0)/(self.h/2.0 + 0.0), 0.1, 50000)
        else:
            gluPerspective(70.0, (self.w + 0.0)/(self.h + 0.0), 0.1, 50000)
        
        glEnable(GL_DEPTH_TEST)
        glDepthRange(-1, self.scale)
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        
        glRotate(globals.firstIsoRotation, 1.0, 0, 0)
        glTranslate(0,0,globals.isoTranslation)
        glRotate(globals.secondIsoRotation, 1.0, 0, 0)
        
    def glTearDown(self):
        glDisable(GL_DEPTH_TEST)


        
    def update(self, dt, otherCarts=None):

        
        startx = self.player.x
        starty = self.player.y

        if self.music == 1 and pygame.time.get_ticks() - self.start_time >= 9500:
            pygame.mixer.music.stop()
            pygame.mixer.music.load(os.path.join('data', "RaceLoop.ogg"))
            pygame.mixer.music.play(-1)
            self.music = 2
            
        done, data = self.player.update(dt)
        self.rotation = self.player.rotation

        if self.computer:
            for opponent in self.computerPlayers:
                opponent.update()

        #just fill the list with tupples of x,y in map coords and it'll show up on the mini map
        #player_list = [(self.x/self.map.scale, self.y/self.map.scale)]
        self.hud.update(self.player_list)
        
        steam = EffectSteam((self.x,self.y,10), spread_v=20)
        self.effects.register(steam)

        hitSomething = False

        """class circle:
            def __init__(self, x, y, rad):
                self.x = x
                self.y = y
                self.radius = rad
                self.mass = 1000.0
                self.xv = 0
                self.yv = 0

        ox = (self.map.startpoint[0] + 100) * self.map.scale
        oy = self.map.startpoint[1] * self.map.scale
        oRad = 8 * self.map.scale
        circ = circle(ox, oy, oRad)"""
        
        
        if otherCarts:
            for cart in otherCarts:
                if circleCollide(self.player, cart):
                    newxv,newyv = circleRespond(self.player, cart)
                    cart.xv, cart.yv = circleRespond(cart, self.player)
                    self.player.xv, self.player.yv = newxv, newyv
                    self.player.slide = True
                    self.player.slideTime = 0.2
                    self.player.momentum = 'n'                    

                    cart.slide = True
                    cart.slideTime = 0.2
                    cart.momentum = 'n'
                    hitSomething = True

                    
        """if circleCollide(self.player, circ):
            self.player.xv, self.player.yv = circleRespond(self.player, circ)
            self.player.x, self.player.y = startx, starty
            self.player.slide = True
            self.player.slideTime = 0.2

            self.player.momentum = 'n'
            hitSomething = True"""
            
            
        for box in self.boxList:
            if squareCollide(self.player, box[0], box[1], box[2], box[3]):
                #print "Hit Square!"
                xm, ym = squareResponse(self.player, box[0], box[1], box[2], box[3])
                self.player.x, self.player.y = startx, starty
                self.x, self.y = startx, starty
                self.player.xv *= xm
                self.player.yv *= ym
                self.player.slide = True
                self.player.slideTime = 0.2
                self.player.momentum = 'n'
                hitSomething = True
        
        if not hitSomething:
            self.x = self.player.x
            self.y = self.player.y
            
            
        if done:
            #self.capture.flush()
            #self.capture.close()
            return ((1,4,4), data)
        else:
            return ((0,0,0), None)
          

        
    def draw(self, dt = 0.0, otherCars=None, otherEffects=None):
        self.glSetup()
        glPushMatrix()
        
        glTranslate(-1.0 * self.player.x,-1.0 * self.player.y,0)

        rotationCenter = (self.player.x,self.player.y)
        glTranslate(rotationCenter[0],rotationCenter[1],0)
        glRotate(self.rotation,0,0,-1)
        glTranslate(-rotationCenter[0],-rotationCenter[1],0)

        self.map.draw(pos=(0,0), width=self.scale, height=self.scale)
        
        
        
        """ Draw the stupid cow """
        #self.cow.drawIso(self.rotation,pos=(self.map.startpoint[0]*self.map.scale, self.map.startpoint[1]*self.map.scale), width=32, height=32)
       
        #for box in self.boxList:
        #    self.cow.drawIso(self.rotation,pos=(box[0], box[1]))
        
        self.player_list = [((self.x/self.map.scale, self.y/self.map.scale), (0,0,1,1))]
        if otherCars != None:
            for car in otherCars:
                rotationImage = car.getImage(self.player.direction,self.playerNum)
                
                self.player_list.append((((car.x/self.map.scale, car.y/self.map.scale)), (1,0,0,1)))
                rotationImage.drawIso(self.rotation,pos=(car.x, car.y), width=80, height=40)
                #car.gdot.draw(pos=(car.x-car.radius,car.y-car.radius),width=car.radius*2, height=car.radius*2)
        
        self.player.draw()
        
        glEnable(GL_DEPTH_TEST)
        for back in self.background:
            img, x, y = back
            img.drawIso(pos=(x,y),mapRotation=self.rotation, height=400, width = 400)
        glDisable(GL_DEPTH_TEST)
        
        if self.computer:
            for opponent in self.computerPlayers:
                rotationImage = opponent.cart.getImage(self.player.direction, self.playerNum)
                rotationImage.drawIso(self.rotation,pos=(opponent.cart.x, opponent.cart.y), width=80, height=40)
                self.player_list.append((((opponent.cart.x/self.map.scale, opponent.cart.y/self.map.scale)), (1,0,0,1)))

            
        self.effects.draw(dt+0.0,self.rotation)
        
        if otherEffects:
            for effect in otherEffects:
                effect.draw(dt+0.0,self.rotation)
        
        #self.steam.draw()
        glPopMatrix()
        self.glTearDown()        
        
        self.hud.draw()


    # Jay is too tired to do real work... so joystick control!
    def handleJoystick(self, side_axis, up_axis):
        #print "Side: ", side_axis, "Up: ", up_axis
        
        if up_axis < -0.3:
            self.player.throttle = 'f'
        elif up_axis > 0.3:
            self.player.throttle = 'r'
            
        if side_axis < -0.3:
            self.player.wheels = 'l'
        elif side_axis > 0.3: 
            self.player.wheels = 'r'
            
            
    def setKeys(self, player=1):
        self.keys = {}
        
        if player == 1:
            self.keys['forward'] = K_UP
            self.keys['back'] = K_DOWN
            self.keys['left'] = K_LEFT
            self.keys['right'] = K_RIGHT
            self.keys['fire'] = K_SPACE
            self.keys['ebreak'] = K_RSHIFT
        else:
            self.keys['forward'] = K_w
            self.keys['back'] = K_s
            self.keys['left'] = K_a
            self.keys['right'] = K_d
            self.keys['fire'] = K_z
            self.keys['ebreak'] = K_LSHIFT
    
        
        
        
    def handleKey(self, type, key):
        
        #self.event_capture.append((pygame.time.get_ticks() - self.start_time, type, key))
        #string = str(pygame.time.get_ticks() - self.start_time) + "   " + str(type) + "   " + str(key) + "\n"
        #self.capture.write(string)
        
        forward = K_UP

        
        
        if type == KEYDOWN:
            if key == self.keys['forward']:
                self.player.throttle = 'f'
            elif key == self.keys['back']:
                self.player.throttle = 'r'
            elif key == self.keys['left']:
                self.player.wheels = 'l'
            elif key == self.keys['right']:
                self.player.wheels = 'r'
            elif key == self.keys['fire']:
                self.player.addFire()
            elif key == self.keys['ebreak']:
                self.player.eBreak = True
            elif key == K_ESCAPE:
                return (1,4,0)

        else:
            if key == self.keys['left'] and self.player.wheels == 'l':
                self.player.wheels = 's'
            elif key == self.keys['right'] and self.player.wheels == 'r':
                self.player.wheels = 's'
            elif key == self.keys['forward'] and self.player.throttle == 'f':
                self.player.throttle = 'n'
                self.player.slide = False
            elif key == self.keys['back'] and self.player.throttle == 'r':
                self.player.throttle = 'n'
                self.player.slide = False
            elif key == self.keys['ebreak']:
                self.player.eBreak = False
                self.player.slide = True

        return (0,0,0)
                