from OpenGL.GL import *
import gutil
import os

import globals

class Image:
    def __init__(self, texname, key=False, mipMap=False,is_surface=False):
        
        if is_surface:
            filename = texname
        else:
            filename = os.path.join('data', texname)
            filename += ".png"

        self.texture, self.width, self.height, self.t_width, self.t_height = gutil.loadImage(filename, key, mipMap, is_surface)
        self.displayList = gutil.createTexDL(self.texture, self.t_width, self.t_height)
        #glDeleteTextures(self.texture)
   

    def __del__(self):

        """glBindTexture(GL_TEXTURE_2D, 0)
        if self.displayList != None:
            gutil.delDL(self.displayList)
            self.displayList = None
        if self.texture != None:
            gutil.delTexture(self.texture)
            self.texture = None"""


    def draw(self, pos=None, width = None, height = None, color=(1,1,1,1), rotation=0, rotationCenter=None, centered=False):
        glPushMatrix()
        glColor4fv(color)

        if pos:
            glTranslate(pos[0],pos[1],0)
            
        if centered:
            glTranslate(-self.width/2,-self.height/2,0)

        if rotation != 0:
                if rotationCenter == None:
                    rotationCenter = (self.width / 2, self.height / 2)
                (w,h) = rotationCenter
                glTranslate(rotationCenter[0],rotationCenter[1],0)
                glRotate(rotation,0,0,-1)
                glTranslate(-rotationCenter[0],-rotationCenter[1],0)

        if width or height:
            if not width:
                width = self.width
            elif not height:
                height = self.height

            glScalef(width/(self.width*1.0), height/(self.height*1.0), 1.0)
                

        glCallList(self.displayList)
        glPopMatrix()
        
    def drawIso(self, mapRotation, pos=None, z=None, width = None, height = None, color=(1,1,1,1), rotation=0, rotationCenter=None, centered=0):
        
        glPushMatrix()


        #adjust z
        if z:
            glTranslate(0,0,z)
            
        glColor4fv(color)



        # move to apropriate x,y
        if pos:
            glTranslate(pos[0],pos[1],0)
            
        #   if pos is middle of cart, adjust apropriately 
        if centered:
            glTranslate(-self.width/2.0,-self.height/2,0)


        # move to middle of cart, spin, move back
        #glTranslate(self.width/2.0,self.height/2,0)
        glRotate(mapRotation * -1.0,0,0,-1)
        #glTranslate(-self.width/2.0,-self.height/2,0)


            
        """if rotation != 0:
                if rotationCenter == None:
                    rotationCenter = (self.width / 2.0, self.height / 2.0)
                (w,h) = rotationCenter
                glTranslate(rotationCenter[0],rotationCenter[1],0)
                glRotate(rotation,0,0,-1)
                glTranslate(-rotationCenter[0],-rotationCenter[1],0)"""


        glRotate(-1.0*(globals.firstIsoRotation + globals.secondIsoRotation), 1.0, 0, 0)


        if width or height:
            if not width:
                width = self.width
            elif not height:
                height = self.height

            glScalef(width/(self.width*1.0), height/(self.height*1.0), 1.0)
                

        glTranslate(-self.width/2.0, 0,0)



        glCallList(self.displayList)
        glPopMatrix()