
import os
import sys
import Config
import pygame
import random
from math import *
from OpenGL.GL import *
import gutil
from Image import Image
from Text import Text

from MapData import MapData

class Map:
    def __init__(self, mapname, needs_path = False):

        # clear display before creating new map.
        glClear(GL_COLOR_BUFFER_BIT)
        
        mappath = os.path.join('maps', mapname)
        #print "Loading Map: ", mapname
        mappath += ".png"
        temp = pygame.image.load(mappath)
        self.terrain = pygame.transform.flip(temp, 0 , 1)
        self.width = self.terrain.get_height() #grab from the image
        self.height = self.terrain.get_width() #grab from the image
        configpath = os.path.join('maps', mapname)
        configpath += ".cfg"
        self.cfg = Config.load(configpath)
        self.fudge = int(self.config("FUDGE_NUMBER"))
        self.map_scale = int(self.config("SCALE"))
        self.road_texture = Image(self.config("ROAD_TEXTURE"), mipMap = True)
        self.rough_texture = Image(self.config("ROUGH_TEXTURE"), mipMap = True)
        self.wall_texture = Image(self.config("WALL_TEXTURE"), mipMap = True)
        self.hazard_texture = Image(self.config("HAZARD_TEXTURE"), mipMap = True)
        self.background_texture = Image(self.config("BACKGROUND"))
        self.underlay_texture = Image(self.config("UNDERLAY"))
        self.sky_texture = Image(self.config("SKY"))
        self.pinger = Image("ping")
        self.text_name = Text(self.config("MAPNAME"))
        x, y = self.config("START_POINT").split(" ")
        self.startpoint = (int(x), self.height-int(y))
        x, y = self.config("FLOW_DIRECTION").split(" ")
        self.startrotation = -90
        self.minimap = None
        

        self.road_color    = (247, 20, 228, 255)
        self.nothing_color = (243, 233, 17, 255)
        self.rough_color   = (61, 198, 120, 255)
        self.hazard_color  = (244, 156, 16, 255)
        self.sp_color      = (0,0,0,255)
        
        for y in range(self.height):
            for x in range(self.width):
                if (self.terrain.get_at((x,y)) == self.sp_color):
                    self.terrain.set_at((x,y),self.road_color)
                    
        #calculate start line location
        set = []
        x = self.startpoint[0]
        (x1, y1) = self.startpoint
        for y in range(self.height):
            if (self.terrain.get_at((x,y)) == self.rough_color):
                if self.is_next_to((x,y), self.nothing_color):
                    set.append((x,y))
                    
        distance = self.width
        i=0
        for index in range(len(set)):
            x, y = set[index]
                
            dist = sqrt((x-x1)**2 + (y-y1)**2)
            if dist < distance:
                distance = dist
                i = index
        self.startline1=set.pop(i)
        
        distance = self.width
        i=0
        for index in range(len(set)):
            x, y = set[index]
                
            dist = sqrt((x-x1)**2 + (y-y1)**2)
            if dist < distance:
                distance = dist
                i = index
        self.startline2=set.pop(i)
        
        # Grab the pre-calculated map data
        if needs_path:
            self.mapData = MapData(mapname)
            self.mapData.getMapData()
        
        self.mapX = self.width
        self.mapY = self.height
        
        (x1,y1) = self.startline1
        (x2,y2) = self.startline2
        x1 -= 1
        x2 -= 1

        self.finishLine = [(x1,y1)]
        for y in range(y2-y1):
            nx = x1
            ny = y1 + y
            self.finishLine.append((nx,ny))
        self.finishLine.append((x2,y2))
        
        self.finish = ((self.startpoint[0]-2,self.startpoint[1]))
        
        #map data
        self.roadpoints = []
        self.roughpoints = []
        self.wallpoints = []
        self.nothingpoints = []
        self.hazardpoints = []
        self.scale = (self.map_scale+0.0)/self.width
        
        # render the map appearance to a display list
        self.displayList = glGenLists(1);
        glNewList(self.displayList,GL_COMPILE);
        glPushMatrix()
        self.buildRoad()
        glPopMatrix()
        glEndList()
        
        self.buildMiniMap()
        
    def config(self, key):
        if key in self.cfg:
            value = self.cfg[key]
        else:
            if key == "FUDGE_NUMBER":
                value = "3"
            elif key == "MAPNAME":
                value = "Unnamed"
            elif key == "SKY":
                value = "dark_sky"
            elif key == "BACKGROUND":
                value = "dark_sky"
            elif key == "UNDERLAY":
                value = "Rotten_Fish_Green"
            elif key == "SCALE":
                value = "10000"
            elif key == "START_POINT":
                value = "0 0"
            elif key == "FLOW_DIRECTION":
                value = "1 0"
            elif key == "ROAD_TEXTURE":
                value = "Rotten_Fish_Dark"
            elif key == "ROUGH_TEXTURE":
                value = "Metal_Rusted_Color"
            elif key == "WALL_TEXTURE":
                value = "stone_wall"
            elif key == "HAZARD_TEXTURE":
                value = "default_hazard"
            elif key == "WALL_HEIGHT":
                value = "40"
            elif key == "WALL_TYPE":
                value = "box"
            else:
                value = None
        return value
               
                
    def buildMiniMap(self):
        #build the surface
        minimap = pygame.Surface((self.width, self.height), flags=pygame.SRCALPHA)
        minimap = minimap.convert_alpha()
        for y in range(self.height):
            for x in range(self.width):  
                minimap.set_at((x,self.height-y-1), (0,0,0,0))
        
        for y in range(self.height):
            for x in range(self.width):
                if self.terrain.get_at((x,self.height-y-1)) == self.road_color or self.terrain.get_at((x,self.height-y-1)) == self.rough_color:
                    minimap.set_at((x,y), (255,255,255,150))

                    
        self.minimap = Image(minimap, is_surface=True)
      
    def buildRoad(self):
        # throw the road in an opengl display list
        
        glDisable(GL_DEPTH_TEST)
        
        fudge = self.fudge
        really_fucking_far = 2000
  
        self.generateBlocks();
 

        #draw the underlay
        glPushMatrix()
        glBindTexture(GL_TEXTURE_2D, self.underlay_texture.texture)
        glBegin(GL_QUADS)
        
        glTexCoord2f(0, 0); glVertex3i(-really_fucking_far,-really_fucking_far,-10)
        glTexCoord2f(0, 1); glVertex3i(-really_fucking_far,really_fucking_far,-10)
        glTexCoord2f(1, 1); glVertex3i(really_fucking_far,really_fucking_far,-10)
        glTexCoord2f(1, 0); glVertex3i(really_fucking_far,-really_fucking_far,-10)
        glEnd()
        glPopMatrix()


        #draw the sky
        glPushMatrix()
        glBindTexture(GL_TEXTURE_2D, self.sky_texture.texture)
        glBegin(GL_QUADS)
        
        glTexCoord2f(0, 0); glVertex3i(-really_fucking_far,-really_fucking_far,500)
        glTexCoord2f(0, 1); glVertex3i(-really_fucking_far,really_fucking_far,500)
        glTexCoord2f(1, 1); glVertex3i(really_fucking_far,really_fucking_far,500)
        glTexCoord2f(1, 0); glVertex3i(really_fucking_far,-really_fucking_far,500)
        glEnd()
        glPopMatrix()
               
        
        #draw the roads
        self.drawBlocks(self.roadpoints, self.road_texture.texture, "Road")
        #draw the rough
        self.drawBlocks(self.roughpoints, self.rough_texture.texture, "Rough")
        #draw the hazard
        self.drawBlocks(self.hazardpoints, self.hazard_texture.texture, "Hazard")
        glEnable(GL_DEPTH_TEST)
        
        #draw the walls
        #print "Generating Walls: ", len(self.wallpoints), " points."
        glPushMatrix()
        glBindTexture(GL_TEXTURE_2D, self.wall_texture.texture)
        glBegin(GL_QUADS)

        wall_height = int(self.config("WALL_HEIGHT"))
        for i in range(len(self.wallpoints)):
            x, y = self.wallpoints[i]
            x = x * fudge
            y = y * fudge
            
            if self.config("WALL_TYPE") == "box":
                A = (x,y,0)                         # bottom left bottom
                B = (x,y+fudge,0)                   # bottom right bottom
                C = (x+fudge,y+fudge,0)             # top right bottom
                D = (x+fudge,y,  0)                 # top left bottom
                E = (x,y,wall_height)              # bottom left top
                F = (x,y+fudge,wall_height)        # bottom right top
                G = (x+fudge,y+fudge,wall_height)  # top right top
                H = (x+fudge,y,wall_height)        # top left top
                
                #glTexCoord2f(0, 0); glVertex3iv(A)  # bottom
                #glTexCoord2f(0, 1); glVertex3iv(B)
                #glTexCoord2f(1, 1); glVertex3iv(C)
                #glTexCoord2f(1, 0); glVertex3iv(D)
                
                glTexCoord2f(0, 0); glVertex3iv(A)  # left
                glTexCoord2f(0, 1); glVertex3iv(E)
                glTexCoord2f(1, 1); glVertex3iv(H)
                glTexCoord2f(1, 0); glVertex3iv(D)
                
                glTexCoord2f(0, 0); glVertex3iv(D)  # back
                glTexCoord2f(0, 1); glVertex3iv(H)
                glTexCoord2f(1, 1); glVertex3iv(G)
                glTexCoord2f(1, 0); glVertex3iv(C)
                
                glTexCoord2f(0, 0); glVertex3iv(B)  # right
                glTexCoord2f(0, 1); glVertex3iv(F)
                glTexCoord2f(1, 1); glVertex3iv(G)
                glTexCoord2f(1, 0); glVertex3iv(C)
                
                glTexCoord2f(0, 0); glVertex3iv(B)  # front
                glTexCoord2f(0, 1); glVertex3iv(F)
                glTexCoord2f(1, 1); glVertex3iv(E)
                glTexCoord2f(1, 0); glVertex3iv(A)
                
                glTexCoord2f(0, 0); glVertex3iv(H) # top
                glTexCoord2f(0, 1); glVertex3iv(E)
                glTexCoord2f(1, 1); glVertex3iv(F)
                glTexCoord2f(1, 0); glVertex3iv(G)
                
            elif self.config("WALL_TYPE") == "fence":
                #if i < len(self.wallpoints):
                #    x1, y1 = self.wallpoints[i+1]
                #else:
                #    x1, y1 = self.wallpoints[0]
                    
                A = (x,y,0)
                B = (x+fudge,y+fudge,0)
                C = (x, y, wall_height)
                D = (x+fudge,y+fudge,wall_height)
                
                glTexCoord2f(0, 0); glVertex3iv(A)
                glTexCoord2f(0, 1); glVertex3iv(B)
                glTexCoord2f(1, 1); glVertex3iv(C)
                glTexCoord2f(1, 0); glVertex3iv(D)                
                
            
            
        glEnd()
        glPopMatrix()
        
        
        #draw the edge of the universe
        glPushMatrix()
        glBindTexture(GL_TEXTURE_2D, self.background_texture.texture)
        glBegin(GL_QUADS)
        
        glTexCoord2f(0, 0); glVertex3i(-really_fucking_far,  -really_fucking_far,  0)
        glTexCoord2f(0, 1); glVertex3i(-really_fucking_far,  -really_fucking_far,  really_fucking_far)
        glTexCoord2f(1, 1); glVertex3i(-really_fucking_far,  really_fucking_far, really_fucking_far)
        glTexCoord2f(1, 0); glVertex3i(-really_fucking_far,  really_fucking_far,0)
        
        glTexCoord2f(0, 0); glVertex3i(-really_fucking_far,  really_fucking_far,  0)
        glTexCoord2f(0, 1); glVertex3i(-really_fucking_far,  really_fucking_far,  really_fucking_far)
        glTexCoord2f(1, 1); glVertex3i(really_fucking_far,  really_fucking_far, really_fucking_far)
        glTexCoord2f(1, 0); glVertex3i(really_fucking_far,  really_fucking_far,0)
        
        glTexCoord2f(0, 0); glVertex3i(really_fucking_far,  -really_fucking_far,  0)
        glTexCoord2f(0, 1); glVertex3i(really_fucking_far,  -really_fucking_far,  really_fucking_far)
        glTexCoord2f(1, 1); glVertex3i(really_fucking_far,  really_fucking_far, really_fucking_far)
        glTexCoord2f(1, 0); glVertex3i(really_fucking_far,  really_fucking_far,0)
        
        glTexCoord2f(0, 0); glVertex3i(-really_fucking_far,  -really_fucking_far,  0)
        glTexCoord2f(0, 1); glVertex3i(-really_fucking_far,  -really_fucking_far,  really_fucking_far)
        glTexCoord2f(1, 1); glVertex3i(really_fucking_far,  -really_fucking_far, really_fucking_far)
        glTexCoord2f(1, 0); glVertex3i(really_fucking_far,  -really_fucking_far,0)
        
        
        glEnd()
        glPopMatrix()
        
        #random debug stuff
        glDisable(GL_DEPTH_TEST)
        
        #self.pinger.draw((self.startpoint[0] + 100, self.startpoint[1]), centered=1)
        #self.pinger.draw((self.flowpoint[0], self.flowpoint[1]), centered=1)
        #self.text_name.draw((self.startpoint[0], self.startpoint[1]),rotation=90)
        
        glDisable(GL_TEXTURE_2D)
        glBegin(GL_LINES)
        glVertex3i(self.startline1[0],self.startline1[1], 20)
        glVertex3i(self.startline2[0],self.startline2[1], 20)     
        glEnd()
        glEnable(GL_TEXTURE_2D)
        
        #end random debug stuff
        
    
    def is_edge(self, loc):
        original = self.terrain.get_at(loc)
        x, y = loc

        if (original != self.terrain.get_at((x-1,y+1))):
            return 1
        elif (original != self.terrain.get_at((x-1,y))):
            return 1
        elif (original != self.terrain.get_at((x-1,y-1))):
            return 1
        elif (original != self.terrain.get_at((x,y-1))):
            return 1
        elif (original != self.terrain.get_at((x+1,y-1))):
            return 1
        elif (original != self.terrain.get_at((x+1,y))):
            return 1
        elif (original != self.terrain.get_at((x+1,y+1))):
            return 1
        elif (original != self.terrain.get_at((x,y+1))):
            return 1
            
        return 0
        
    def is_next_to(self, loc, compare):
        x, y = loc

        if (compare == self.terrain.get_at((x-1,y+1))):
            return 1
        elif (compare == self.terrain.get_at((x-1,y))):
            return 1
        elif (compare == self.terrain.get_at((x-1,y-1))):
            return 1
        elif (compare == self.terrain.get_at((x,y-1))):
            return 1
        elif (compare == self.terrain.get_at((x+1,y-1))):
            return 1
        elif (compare == self.terrain.get_at((x+1,y))):
            return 1
        elif (compare == self.terrain.get_at((x+1,y+1))):
            return 1
        elif (compare == self.terrain.get_at((x,y+1))):
            return 1
            
        return 0
        
    
    def generateBlocks(self):
        fudge = self.fudge
        
        #find bounding edges
        min_y = self.height
        min_x = self.width
        max_y = 0
        max_x = 0
        
        #print "Processing map blocks..."
        for y in range(self.height):
            for x in range(self.width):
                if self.terrain.get_at((x,y)) != self.nothing_color:
                    if x > max_x:
                        max_x = x
                    if x < min_x:
                        min_x = x
                    if y > max_y:
                        max_y = y
                    if y < min_y:
                        min_y = y
        
        if min_y > 0 + fudge:
            min_y -= fudge
        if min_x > 0 + fudge:
            min_x -= 1
        if max_y < self.height - fudge:
            max_y += fudge
        if max_x < self.width - fudge:
            max_x += fudge
            
        range_y = int(1+ (max_y - min_y)/fudge)
        range_x = int(1+ (max_x - min_x)/fudge)
        
        #build the blocks
        for y1 in range(int(min_y/fudge)-1, int(min_y/fudge)+range_y+1):
            for x1 in range(int(min_x/fudge)-1, int(min_x/fudge)+range_x+1):
                #count how many of each type are inside each block
                road = 0
                nothing = 0
                rough = 0
                wall = 0
                hazard = 0
                for y2 in range(y1*fudge, y1*fudge+fudge):
                    for x2 in range(x1*fudge, x1*fudge+fudge):
                        map_color = self.terrain.get_at((x2,y2))
                 
                        if(map_color == self.nothing_color):
                            nothing += 1
                        elif(map_color == self.road_color):
                            road += 1
                        elif(map_color == self.hazard_color):
                            hazard += 1                            
                        elif(map_color == self.rough_color):
                            rough += 1
                            if self.is_next_to((x2,y2), self.nothing_color):
                                wall += 1
                if wall > 0:
                    self.wallpoints.append((x1,y1))
                elif nothing > road and nothing > rough:
                    self.nothingpoints.append((x1,y1))
                elif road > nothing and road > rough:
                    self.roadpoints.append((x1,y1))
                elif rough > road and rough > nothing:
                    self.roughpoints.append((x1,y1))
                elif hazard > road and hazard > rough:
                    self.hazardpoints.append((x1,y1))
                else:
                    """"""
                    #this should never happen, but for some reason does when you don't use a prime for fudge
                    #print "omg! so screwed!"
        
        self.min_y = min_y
        self.max_y = max_y
        self.min_x = min_x
        self.max_x = max_x

    def draw(self, pos=None, width = None, height = None, color=(1,1,1,1), rotation=0, rotationCenter=None):
        glPushMatrix()
        glColor4fv(color)

        if pos:
            glTranslate(pos[0],pos[1],0)

        if rotation != 0:
                if rotationCenter == None:
                    rotationCenter = (self.width / 2, self.height / 2)
                (w,h) = rotationCenter
                glTranslate(rotationCenter[0],rotationCenter[1],0)
                glRotate(rotation,0,0,-1)
                glTranslate(-rotationCenter[0],-rotationCenter[1],0)

        if width or height:
            if not width:
                width = self.width
            elif not height:
                height = self.height

            glScalef(width/(self.width*1.0), height/(self.height*1.0), 1.0)
                

        glCallList(self.displayList)
        glPopMatrix()
        
    def drawIso(self, pos=None, width = None, height = None, color=(1,1,1,1), rotation=0, rotationCenter=None):
        glPushMatrix()
        
    def tileType(self, pos, need_to_scale=False):
        x, y = pos
        
        if need_to_scale:
            x = int(x/self.scale)
            y = int(y/self.scale)
            
        map_color = self.terrain.get_at((x,y))
        if (map_color == self.nothing_color):
            return "nothing"
        elif(map_color == self.road_color):
            return "road"
        elif(map_color == self.hazard_color):
            return "hazard"
        elif(map_color == self.rough_color):
            if self.is_next_to((x,y), self.nothing_color):
                return "wall"
            return "rough"
        
        return "unknown"
        
    def drawBlocks(self, points, texture, name):
        fudge = self.fudge
        #print "Generating", name,": ", len(points), " points."
        glPushMatrix()
        glBindTexture(GL_TEXTURE_2D, texture)
        glBegin(GL_QUADS)

        for i in range(len(points)):
            x, y = points[i]
            x = x * fudge
            y = y * fudge
            glTexCoord2f(0, 0); glVertex3i(x,  y,  0)
            glTexCoord2f(0, 1); glVertex3i(x,  y+fudge,0)
            glTexCoord2f(1, 1); glVertex3i(x+fudge,y+fudge,0)
            glTexCoord2f(1, 0); glVertex3i(x+fudge,y,  0)

        glEnd()
        glPopMatrix()