import os
import pygame
import Config

from Image import Image
from PathFinder import PathFinder
from MapData import MapData
from math import *
from Map import Map

class MapParser:
    
    def __init__(self, mapname):
    
        self.mapName = mapname
        
        
        self.N  = 1
        self.NE = 2
        self.E  = 3
        self.SE = 4
        self.S  = 5
        self.SW = 6
        self.W  = 7
        self.NW = 8
    
    def parseMap(self):

        mappath = os.path.join('maps', self.mapName)
        print "Loading Map: ", self.mapName
        mappath += ".png"
        temp = pygame.image.load(mappath)
        self.terrain = temp
        self.width = self.terrain.get_height() #grab from the image
        self.height = self.terrain.get_width() #grab from the image
        configpath = os.path.join('maps', self.mapName)
        configpath += ".cfg"
        self.cfg = Config.load(configpath)
        self.fudge = int(self.config("FUDGE_NUMBER"))
        self.map_scale = int(self.config("SCALE"))
        x, y = self.config("START_POINT").split(" ")
        self.startpoint = (int(x), self.height-int(y))
        x, y = self.config("FLOW_DIRECTION").split(" ")
        self.startrotation = -90
        self.minimap = None
        #self.terrain = mymap.terrain
        self.terrain = pygame.Surface((self.width, self.height), flags=pygame.SRCALPHA)
        
        for y in range(self.height):
            for x in range(self.width):
                color = temp.get_at((x,y))
                self.terrain.set_at((x,self.height-y), color)

        self.road_color    = (247, 20, 228, 255)
        self.nothing_color = (243, 233, 17, 255)
        self.rough_color   = (61, 198, 120, 255)
        self.hazard_color  = (244, 156, 16, 255)
        self.sp_color      = (0,0,0,255)

        #calculate start line location
        set = []
        x = self.startpoint[0]
        (x1, y1) = self.startpoint
        for y in range(self.height):
            print self.terrain.get_at((x,y))
            if (self.terrain.get_at((x,y)) == self.rough_color):
                if self.is_next_to((x,y), self.nothing_color):
                    set.append((x,y))
                    
        distance = self.width
        i=0
        for index in range(len(set)):
            x, y = set[index]
                
            dist = sqrt((x-x1)**2 + (y-y1)**2)
            if dist < distance:
                distance = dist
                i = index
        self.startline1=set.pop(i)
        
        distance = self.width
        i=0
        for index in range(len(set)):
            x, y = set[index]
                
            dist = sqrt((x-x1)**2 + (y-y1)**2)
            if dist < distance:
                distance = dist
                i = index
        self.startline2=set.pop(i)
        
        
        
        #map data
        self.roadpoints = []
        self.roughpoints = []
        self.wallpoints = []
        self.nothingpoints = []
        self.hazardpoints = []
        self.scale = (self.map_scale+0.0)/self.width

        
        # Define the cost for moving to each type of terrain.  Used by path finder
        self.MAX_COST = 99999999
        self.ROAD_COST = 2
        self.OPTIMAL_COST = 1
        self.ROUGH_COST = 5
        self.HAZARD_COST = 100
        
        self.mapX = self.width
        self.mapY = self.height
        
        # See if we are facing the "wrong" way
        """
        (xx,yy) = self.flowpoint
        dx = -1
        print "STarting at ", (self.startpoint[0],self.startpoint[1]), " and looking at ", (xx,yy)
        if ((xx - self.startpoint[0]) > 0):
            dx = 1
        """
        
        (x1,y1) = self.startline1
        (x2,y2) = self.startline2
        x1 -= 1
        x2 -= 1
        
        if (y2 < y1):
            (tx,ty) = (x1,y1)
            (x1,y1) = (x2,y2)
            (x2,y2) = (tx,ty)
            
        print "start line = ", (x1,y1), (x2,y2)
        self.finishLine = [(x1,y1)]

        for y in range(y2-y1+6):
            nx = x1
            ny = y1 - 3 + y
            self.finishLine.append((nx,ny))
        self.finishLine.append((x2,y2))
        
        self.finish = ((self.startpoint[0]-2,self.startpoint[1]))
        print self.finishLine

        self.scale = (self.map_scale+0.0)/self.width
        
        self.pathFinder = PathFinder(self.getGraph())
        self.sp = None
        self.path(self.startpoint[0], self.startpoint[1], self.finish)
        
        
        self.directions = self.getDirections()
        
        self.nicePath = []
        self.nicePath = self.getNicePath()
        
        print "SHORTEST PATH:"
        print self.sp
        print "\n\n"
        print "DIRECTIONS:"
        print self.directions
        
        data = MapData(self.mapName)
        data.shortestPath = self.sp
        data.pathDirections = self.directions
        data.nicePath = self.nicePath
        
        data.pickle()
        
    def config(self, key):
        if key in self.cfg:
            value = self.cfg[key]
        else:
            if key == "FUDGE_NUMBER":
                value = "3"
            elif key == "MAPNAME":
                value = "Unnamed"
            elif key == "SKY":
                value = "dark_sky"
            elif key == "BACKGROUND":
                value = "dark_sky"
            elif key == "UNDERLAY":
                value = "Rotten_Fish_Green"
            elif key == "SCALE":
                value = "10000"
            elif key == "START_POINT":
                value = "0 0"
            elif key == "FLOW_DIRECTION":
                value = "1 0"
            elif key == "ROAD_TEXTURE":
                value = "Rotten_Fish_Dark"
            elif key == "ROUGH_TEXTURE":
                value = "Metal_Rusted_Color"
            elif key == "WALL_TEXTURE":
                value = "stone_wall"
            elif key == "HAZARD_TEXTURE":
                value = "default_hazard"
            elif key == "WALL_HEIGHT":
                value = "40"
            elif key == "WALL_TYPE":
                value = "box"
            else:
                value = None
        return value
        
    def path(self, x, y, (fx, fy)):
        
        self.i = 0
        self.step = 0
        
        self.sp = self.pathFinder.shortestPath(x, y, fx, fy)        
        
        
    def getGraph(self):
        
        G = {}
        
        for y in range(self.mapY):
            for x in range(self.mapX):
                temp = {}
                
                xx = x - 1
                yy = y - 1              
                cost = self.cost(xx,yy)
                if (cost != self.MAX_COST):
                    temp[(xx,yy)] = self.cost(xx,yy)
                    
                xx = x - 1
                yy = y - 0              
                cost = self.cost(xx,yy)
                if (cost != self.MAX_COST):
                    temp[(xx,yy)] = self.cost(xx,yy)
                    
                xx = x - 1
                yy = y + 1              
                cost = self.cost(xx,yy)
                if (cost != self.MAX_COST):
                    temp[(xx,yy)] = self.cost(xx,yy)
                    
                xx = x - 0
                yy = y - 1              
                cost = self.cost(xx,yy)
                if (cost != self.MAX_COST):
                    temp[(xx,yy)] = self.cost(xx,yy)
                    
                xx = x - 0
                yy = y + 1              
                cost = self.cost(xx,yy)
                if (cost != self.MAX_COST):
                    temp[(xx,yy)] = self.cost(xx,yy)
                    
                xx = x + 1
                yy = y - 1              
                cost = self.cost(xx,yy)
                if (cost != self.MAX_COST):
                    temp[(xx,yy)] = self.cost(xx,yy)
                    
                xx = x + 1
                yy = y - 0              
                cost = self.cost(xx,yy)
                if (cost != self.MAX_COST):
                    temp[(xx,yy)] = self.cost(xx,yy)
                    
                xx = x + 1
                yy = y + 1              
                cost = self.cost(xx,yy)
                if (cost != self.MAX_COST):
                    temp[(xx,yy)] = self.cost(xx,yy)
                    
                G[(x,y)] = temp
                
        return G
                
    def getDirections(self):
        
        directions = []
        
        directions.append(self.N)
        
        for i in range(1,len(self.sp)):
        
            j = i + 1
            if (j >= len(self.sp)): 
                j = 0
                
            (x1, y1) = self.sp[i]
            (x2, y2) = self.sp[j]
            
            print (x1, y1), (x2, y2)
            dir = self.getFacingDirection(x1, y1, x2, y2)
            directions.append(dir)
            
        return directions
        
    def getFacingDirection(self, x1, y1, x2, y2):
        
        dx = x2 - x1
        dy = y2 - y1
        
        if (dx < 0):
            if (dy < 0):
                dir = self.SE
            elif (dy == 0):
                dir = self.S
            else:
                dir = self.SW
        elif (dx > 0):
            if (dy < 0):
                dir = self.NE
            elif (dy == 0):
                dir = self.N
            else:
                dir = self.NW
        else:
            if (dy < 0):
                dir = self.E
            elif (dy > 0):
                dir = self.W
                
        return dir
        
    def cost(self, x, y):
       
        if (x < 0 or x >= self.mapX):
            return self.MAX_COST
        
        if (y < 0 or y >= self.mapY):
            return self.MAX_COST
            
        if ((x,y) in self.finishLine):
            return self.MAX_COST
            
        terrain = self.terrain.get_at((x,y))
        
        if (terrain == self.road_color):
            return self.ROAD_COST
            
        elif (terrain == self.rough_color):
            return self.ROUGH_COST
            
        elif (terrain == self.hazard_color):
            return self.HAZARD_COST
            
        elif (terrain == self.nothing_color):
            return self.MAX_COST
            
        elif (terrain == self.sp_color):
            return self.OPTIMAL_COST
            
        else:
            return self.MAX_COST
            
    def getNicePath(self):
        
        nicePath = []
        
        (nx, ny) = self.sp[0]
        
        for i in range(len(self.sp)):
            
            (nx, ny) = self.findMiddleOfRoad(i, nx, ny)
            nicePath.append((nx,ny))
        
        return nicePath
    
    def findMiddleOfRoad(self, i, x, y):
        
        dx = dy = 0
        
        dir = self.directions[i]
        
        if (dir == self.N):
            dx = 1
            dy = 0
        elif (dir == self.NW):
            dx = 1
            dy = 1
        elif (dir == self.NE):
            dx = 1
            dy = -1
        elif (dir == self.S):
            dx = -1
            dy = 0
        elif (dir == self.SW):
            dx = -1
            dy = 1
        elif (dir == self.SE):
            dx = -1
            dy = -1
        elif (dir == self.W):
            dx = 0
            dy = 1
        elif (dir == self.E):
            dx = 0
            dy = -1

            
        """
        if (dir in (self.S, self.N)):
            
            for ny in range(y+1, self.height):
                if (self.terrain.get_at((x,ny)) != self.nothing_color):
                    dy2 += 1
                else:
                    break

            for ny in range(y-1, 0, -1):
                if (self.terrain.get_at((x,ny)) != self.nothing_color):
                    dy1 += 1
                else:
                    break
         
        elif (dir in (self.W, self.E)):
        
            for nx in range(x+1, self.width):
                if (self.terrain.get_at((nx,y)) != self.nothing_color):
                    dx2 += 1
                else:
                    break

            for nx in range(x-1, 0, -1):
                if (self.terrain.get_at((nx,y)) != self.nothing_color):
                    dx1 += 1
                else:
                    break
                    
        print (dx1, dx2), (dy1, dy2)

        
        xx = x + (dx2 + dx1) / 2
        yy = y + (dy2 - dy1) / 2
        print "Converting ", (x,y), " to ", (xx,yy)
        """
        xx = x + dx
        yy = y + dy
        return (xx,yy)
        
        
    def is_edge(self, loc):
        original = self.terrain.get_at(loc)
        x, y = loc

        if (original != self.terrain.get_at((x-1,y+1))):
            return 1
        elif (original != self.terrain.get_at((x-1,y))):
            return 1
        elif (original != self.terrain.get_at((x-1,y-1))):
            return 1
        elif (original != self.terrain.get_at((x,y-1))):
            return 1
        elif (original != self.terrain.get_at((x+1,y-1))):
            return 1
        elif (original != self.terrain.get_at((x+1,y))):
            return 1
        elif (original != self.terrain.get_at((x+1,y+1))):
            return 1
        elif (original != self.terrain.get_at((x,y+1))):
            return 1
            
        return 0
        
    def is_next_to(self, loc, compare):
        x, y = loc

        if (compare == self.terrain.get_at((x-1,y+1))):
            return 1
        elif (compare == self.terrain.get_at((x-1,y))):
            return 1
        elif (compare == self.terrain.get_at((x-1,y-1))):
            return 1
        elif (compare == self.terrain.get_at((x,y-1))):
            return 1
        elif (compare == self.terrain.get_at((x+1,y-1))):
            return 1
        elif (compare == self.terrain.get_at((x+1,y))):
            return 1
        elif (compare == self.terrain.get_at((x+1,y+1))):
            return 1
        elif (compare == self.terrain.get_at((x,y+1))):
            return 1
            
        return 0
        
