import os
import sys
import gutil
import pygame
import Config
from pygame.locals import *
from OpenGL.GL import *
from Text import Text
from Image import Image
from OpenGL.GLU import *

class TimeTrial:

    def __init__(self, screen, lap_data, map_name):
        self.w = screen[0]
        self.h = screen[1]
        high = 999999999999999999

        configpath = os.path.join('maps', map_name)
        configpath += ".cfg"
        cfg = Config.load(configpath)
        name = cfg["MAPNAME"]
        
        configpath = os.path.join('data', 'high_score.txt')
        highscores = Config.load(configpath)
        
        black = (255,255,255,255)
        self.background = Image("Racer1")
        self.corner = Text("Esc to exit.   ",color=(0,0,0,255),fontsize = 40)
        self.title = Text(name,color=(0,0,0,255),fontsize = 50)
        
        self.times = []
        previous = 0
        for i in lap_data:
            lap, time = i
            time -= previous
            
            string = "Lap: " + str(lap) + " Time: %0.2f" % (time/1000.0)
            self.times.append(Text(string, color=black,fontsize = 30))
            previous = i[1]
                
        x, total = lap_data[len(lap_data)-1]
        string = "  Total Time: %0.2f" % (total/1000.0)
        
        if map_name in highscores:
            high = int(highscores[map_name])
        
        if high > total:
            string += " New Best Time!"
            self.times.append(Text(string, color=black,fontsize = 30))
            high = total
            highscores[map_name] = str(high)
            Config.save(highscores, configpath)
        else:
            self.times.append(Text(string, color=black,fontsize = 30))
            string = "  Best Time: %0.2f" % (high/1000.0)
            self.times.append(Text(string, color=black,fontsize = 30))
        
        self.options = []
        self.options.append(((1,4,2), Text("Repeat Course", color=black,fontsize = 30)))
        self.options.append(((1,4,0), Text("New Course", color=black,fontsize = 30)))
        self.options.append(((1,1,0), Text("Main Menu", color=black,fontsize = 30)))
        self.position = 0
        
        
    #draws the list to the screen on a vertical offset   
    def draw(self, pos=(0,300), color=(1,1,1,1), screen_width=800):
        glDisable(GL_DEPTH_TEST)
        glMatrixMode(GL_PROJECTION)
        glPushMatrix()
        glLoadIdentity()
        gluOrtho2D(0, self.w, 0, self.h)
        glMatrixMode(GL_MODELVIEW)
        glPushMatrix()
        glColor4fv(color)
        y = self.h-self.title.height

        #self.text.draw(pos)
        self.background.draw(width = self.w, height = self.h)
        self.corner.draw(pos = (self.w-self.corner.width, 20))
        self.title.draw(pos = (0,y))
        y -= self.title.height
        
        for item in self.times:
            y -= item.height
            menu_color = (0,0,0,255)
            item.draw(pos = (20, y), color = menu_color)   

        y -= self.title.height
        
        for i in range(len(self.options)):
            x, item = self.options[i]
            y -= item.height
            menu_color = (0,0,0,255)
            if i == self.position:
                menu_color = (0,0,255,255)
            item.draw(pos = (20, y), color = menu_color)

        glPopMatrix()
        glMatrixMode(GL_PROJECTION)
        glPopMatrix()
        glMatrixMode(GL_MODELVIEW)
        glEnable(GL_DEPTH_TEST)
        
    def handleKey(self, type, key):
        if type == KEYDOWN:
            if key == K_ESCAPE:
                return (1,1,0)
            elif key == K_UP:
                self.position -= 1
                if self.position < 0:
                    self.position = len(self.options) - 1
            elif key == K_DOWN:
                self.position += 1
                if self.position > len(self.options) - 1:
                    self.position = 0
            elif key == K_SPACE or key == K_RETURN:
                x, y = self.options[self.position]
                return (x)
                
        return (0,0,0)
                