#!/usr/bin/python

import os
import gutil
import pygame
import Config
from pygame.locals import *

from Engine import Engine
from Splash import Splash
from Options import Options
from MainMenu import MainMenu
from MapSelect import MapSelect
from TimeTrial import TimeTrial
from CreditRoll import CreditRoll
from MultiplayerWin import MultiplayerWin
from OpenGL.GL import *


#from Image import Image
#from Text import Text

deltafactor=1000.0

def main():
    #load all the configuration options
    cfg = Config.load("options.cfg")
    w = int(cfg["SCREEN_WIDTH"])
    h = int(cfg["SCREEN_HEIGHT"])
    fullscreen = int(cfg["SCREEN_MODE"])
    state = int(cfg["INITAL_STATE"])
    substate = int(cfg["INITAL_SUB_STATE"])
    music_volume = float(cfg["SOUND_MUSIC_VOLUME"])/100.0
    ai = int(cfg["AI"])
    screen_info = (w,h)
    map = ""
    results = ""
    mode = ""
    winner = ""

    #setup paramaters to run
    pygame.init()
    pygame.display.set_caption("Steamrollers")
    icon = pygame.image.load(os.path.join('data', 'icon.bmp'))
    pygame.display.set_icon(icon)
    gutil.initializeDisplay(w, h, fullscreen)
    pygame.mixer.init()
    pygame.mixer.music.set_volume(music_volume)
    done = False
    clock = pygame.time.Clock()
    clock.tick()    
    lasttick = pygame.time.get_ticks()
    pygame.mouse.set_visible(False)

    joystick = None
    #if availible, set up joystick
    if pygame.joystick.get_count():
        #print "Joystick found.", pygame.joystick.get_count()
        pygame.joystick.init()
        joystick = pygame.joystick.Joystick(0)
        joystick.init()
        

    #setup game state
    #-1 = exit
    # 0 = splashscreen
    # 1 = main menu
    # 2 = options
    # 3 = credits
    # 4 = start game
    state_ = 0
    substate_ = 0
    change_state = 0
    
    splash = None
    options = None
    mapselect = None
    mainmenu = None
    credits = None
    engine = None
    times = None
    multiwin = None

    while not done:
        glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)

        now = pygame.time.get_ticks()
        delta = (now - lasttick) / deltafactor
        lasttick = now
        
        #Splash Screen
        if (state == 0):
            if (substate == 0):
                splash = Splash(screen = screen_info)
                substate = 1
            elif (substate == 1):
                splash.draw()
        #Main Menu
        elif (state == 1):
            if (substate == 0):
                mainmenu = MainMenu(screen = screen_info)
                substate = 1
            elif (substate == 1):
                mainmenu.draw()
        #Options
        elif (state == 2):
            if (substate == 0):
                options = Options(screen = screen_info)
                substate = 1
            elif (substate == 1):
                options.draw()
        #Credits
        elif (state == 3):
            if (substate == 0):
                credits = CreditRoll(screen = screen_info,speed=3)
                substate = 1
            elif (substate == 1):
                credits.update(delta)
                credits.draw(pos=(0,0),screen_width=w)
        #Game
        elif (state == 4):
            #Map Selecter
            if (substate == 0):
                mapselect = MapSelect(screen = screen_info)
                substate = 1
            if (substate == 1):
                mapselect.draw()
            #Two Player
            if (substate == 2):
                engine = Engine(screen = screen_info, map_name = map, mode = 't',player=1)
                engine2 = Engine(screen = screen_info, map_name = map, mode ='b',player=2)
                substate = 3
            elif (substate == 3):
                x, results = engine.update(delta, otherCarts=(engine2.player,))
                if x[0]:
                    change_state, state_, substate_ = x
                    substate_ = 10
                    winner = "Player 1"
                x, results = engine2.update(delta)
                if x[0]:
                    change_state, state_, substate_ = x
                    substate_ = 10
                    winner = "Player 2"
                engine.draw(delta, otherCars=(engine2.player,), otherEffects=(engine2.effects,))
                glViewport(0,0,screen_info[0],screen_info[1])
                engine2.draw(delta, otherCars=(engine.player,), otherEffects=(engine.effects,))
                glViewport(0,0,screen_info[0],screen_info[1])
            #Display Results
            elif (substate == 4):
                times = TimeTrial(screen = screen_info, lap_data=results, map_name = map)
                substate = 5
            elif (substate == 5):
                times.draw()
            #Time Trial
            elif (substate == 6):
                engine = Engine(screen = screen_info, map_name = map)
                substate = 7
            elif (substate == 7):
                x, results = engine.update(delta)
                change_state, state_, substate_ = x
                engine.draw(delta)
            #One Player
            elif (substate == 8):
                engine = Engine(screen = screen_info, map_name = map, computer=ai)
                substate = 9
            elif (substate == 9):
                x, results = engine.update(delta)
                change_state, state_, substate_ = x
                engine.draw(delta)
            #Multiple Winners
            elif (substate == 10):
                multiwin = MultiplayerWin(screen = screen_info, map_name = map, winner = winner)
                substate = 11
            elif (substate == 11):
                multiwin.draw()
        
        pygame.display.flip()
        
        eventlist = pygame.event.get()
        for event in eventlist:
            if event.type == QUIT:
                done = True
            elif event.type == KEYDOWN or event.type == KEYUP:
                if (state == 0):
                    change_state, state_, substate_ = splash.handleKey(event.type, event.key)
                elif (state == 1):
                    x, mode = mainmenu.handleKey(event.type, event.key)
                    change_state, state_, substate_ = x
                elif (state == 2):
                    change_state, state_, substate_ = options.handleKey(event.type, event.key)
                elif (state == 3):
                    change_state, state_, substate_ = credits.handleKey(event.type, event.key)
                elif (state == 4):
                    if (substate == 1):
                        change_state, state_, substate_, map = mapselect.handleKey(event.type, event.key)
                        if change_state and state_ == 4:
                            substate_ = mode
                    elif (substate == 3):
                        change_state, state_, substate_ = engine.handleKey(event.type, event.key)
                        engine2.handleKey(event.type, event.key)
                    elif (substate == 5):
                        change_state, state_, substate_ = times.handleKey(event.type, event.key)
                    elif (substate == 7) or (substate == 9):
                        change_state, state_, substate_ = engine.handleKey(event.type, event.key)
                    elif (substate == 11):
                        change_state, state_, substate_ = times.handleKey(event.type, event.key)
                        
            elif event.type == JOYAXISMOTION:
                if (state == 4) and ((substate == 3) or (substate == 7) or (substate == 9)):
                    engine.handleJoystick(joystick.get_axis(0), joystick.get_axis(1))
                    
        if change_state:
            #print "From: ", (state, substate), " To: ", (state_, substate)
            state = state_
            substate = substate_
            change_state = 0
            state_ = 0
            substate_ = 0
                    
        if state == -1:
            done = True
            

if __name__ == '__main__':
    main()
